@@ -1,5 +1,5 @@
 /* This is -*- C -*- */
-/* $Id: guppi-delimited-importer.c,v 1.5 2000/02/21 18:36:00 trow Exp $ */
+/* $Id: guppi-delimited-importer.c,v 1.6 2000/03/04 18:55:24 trow Exp $ */
 
 /*
  * guppi-delimited-importer.c
@@ -26,6 +26,7 @@
  */
 
 #include <ctype.h>
+#include <guppi-progress.h>
 #include <guppi-stream.h>
 #include <guppi-string.h>
 #include <guppi-seq-string.h>
@@ -970,7 +971,7 @@ guppi_delimited_importer_import(GuppiDataImporter* imp)
 {
   GuppiDelimitedImporter* gdi;
   GuppiStream* str;
-  gint i, j, line_no;
+  gint i, j, line_no, est_line_count;
   const gchar* line;
   GuppiData** datav;
   GuppiData** dv2;
@@ -1022,6 +1023,7 @@ guppi_delimited_importer_import(GuppiDataImporter* imp)
 
   
   line_no=0;
+  est_line_count = guppi_stream_estimated_number_of_lines(str);
   while ((line = guppi_stream_get_sanitized_line(str, line_no))) {
 
     if (line && !guppi_delimited_importer_line_skip(gdi, line, line_no)) {
@@ -1068,12 +1070,21 @@ guppi_delimited_importer_import(GuppiDataImporter* imp)
     }
     ++line_no;
 
-    if (line_no % 1000 == 0) {
+    if (line_no % 100 == 0) {
+
+      if (est_line_count > 0)
+	guppi_progress_set_percentage(MIN(line_no / (double)est_line_count,1));
+      else
+	guppi_progress_show_activity();
+
       /* In a big import, it is nice for our UI not to freeze */
       while (gtk_events_pending())
 	gtk_main_iteration();
     }
   }
+
+  if (est_line_count > 0)
+    guppi_progress_set_percentage(0);
   
   /* "Collapse" datav */
   /*
@@ -1126,4 +1137,4 @@ guppi_delimited_importer_import(GuppiDataImporter* imp)
 
 
 
-/* $Id: guppi-delimited-importer.c,v 1.5 2000/02/21 18:36:00 trow Exp $ */
+/* $Id: guppi-delimited-importer.c,v 1.6 2000/03/04 18:55:24 trow Exp $ */
