@@ -1,5 +1,5 @@
 /* This is -*- C -*- */
-/* $Id: guppi-scatter-element.c,v 1.4 2000/02/28 05:03:08 trow Exp $ */
+/* $Id: guppi-scatter-element.c,v 1.6 2000/03/09 22:59:46 trow Exp $ */
 
 /*
  * guppi-scatter-element.c
@@ -126,19 +126,22 @@ guppi_scatter_element_class_init(GuppiScatterElementClass* klass)
   object_class->destroy = guppi_scatter_element_destroy;
   object_class->finalize = guppi_scatter_element_finalize;
 
-  guppi_plot_toolkit_set_button_tool(pe_klass->default_toolkit,
+  guppi_plot_toolkit_set_button_tool(&pe_klass->default_toolkit,
 				     2, GDK_SHIFT_MASK,
 				     guppi_scatter_tool_frame_brush(0));
 
   for (i=1; i<=3; ++i) 
-    guppi_plot_toolkit_set_button_tool(pe_klass->default_toolkit,
+    guppi_plot_toolkit_set_button_tool(&pe_klass->default_toolkit,
 				       i, GDK_CONTROL_MASK,
 				       guppi_scatter_tool_radius_brush(i,10*i));
 
-  guppi_plot_toolkit_set_button_tool(pe_klass->default_toolkit,
+  guppi_plot_toolkit_set_button_tool(&pe_klass->default_toolkit,
 				     1, GDK_SHIFT_MASK | GDK_CONTROL_MASK,
 				     guppi_scatter_tool_home_view());
 
+  guppi_plot_toolkit_set_button_tool(&pe_klass->default_toolkit,
+				     2, GDK_SHIFT_MASK | GDK_CONTROL_MASK,
+				     guppi_scatter_tool_drag_point());
 }
 
 static void
@@ -191,4 +194,4 @@ guppi_scatter_element_new_with_state(GuppiScatterState* s)
 
 
 
-/* $Id: guppi-scatter-element.c,v 1.4 2000/02/28 05:03:08 trow Exp $ */
+/* $Id: guppi-scatter-element.c,v 1.6 2000/03/09 22:59:46 trow Exp $ */
