@@ -1,4 +1,4 @@
-/* $Id: guppi-scatter-item.c,v 1.8 2000/02/28 05:06:17 trow Exp $ */
+/* $Id: guppi-scatter-item.c,v 1.11 2000/03/09 22:59:46 trow Exp $ */
 
 /*
  * guppi-scatter-item.c
@@ -41,8 +41,8 @@ guppi_scatter_item_set_arg(GtkObject* obj, GtkArg* arg, guint arg_id)
 
   switch (arg_id) {
   case ARG_STATE:
-    guppi_scatter_item_set_state(si,
-				 GUPPI_SCATTER_STATE(GTK_VALUE_POINTER(*arg)));
+    guppi_canvas_item_set_state(GUPPI_CANVAS_ITEM(si),
+				GUPPI_ITEM_STATE(GTK_VALUE_POINTER(*arg)));
     break;
   default:
     return;
@@ -55,21 +55,36 @@ guppi_scatter_item_set_arg(GtkObject* obj, GtkArg* arg, guint arg_id)
 static void
 guppi_scatter_item_reset_x_bounds(GuppiScatterItem* si)
 {
+  GuppiScatterState* state =
+    GUPPI_SCATTER_STATE(guppi_canvas_item_state(GUPPI_CANVAS_ITEM(si)));
+  double a, b, w;
+
   g_return_if_fail(si != NULL);
-  if (si->state && si->state->x_data)
-    guppi_canvas_item_set_x_bounds(GUPPI_CANVAS_ITEM(si),
-				   guppi_seq_scalar_min(si->state->x_data),
-				   guppi_seq_scalar_max(si->state->x_data));
+
+  if (state && state->x_data) {
+    a = guppi_seq_scalar_min(state->x_data);
+    b = guppi_seq_scalar_max(state->x_data);
+    w = (b-a)*0.025;
+    guppi_canvas_item_set_x_bounds(GUPPI_CANVAS_ITEM(si), a-w, b+w);
+    guppi_canvas_item_home_view(GUPPI_CANVAS_ITEM(si));
+  }
 }
 
 static void
 guppi_scatter_item_reset_y_bounds(GuppiScatterItem* si)
 {
+  GuppiScatterState* state =
+    GUPPI_SCATTER_STATE(guppi_canvas_item_state(GUPPI_CANVAS_ITEM(si)));
+  double a, b, w;
+
   g_return_if_fail(si != NULL);
-  if (si->state && si->state->y_data)
-    guppi_canvas_item_set_y_bounds(GUPPI_CANVAS_ITEM(si),
-				   guppi_seq_scalar_min(si->state->y_data),
-				   guppi_seq_scalar_max(si->state->y_data));
+  if (state && state->y_data) {
+    a = guppi_seq_scalar_min(state->y_data);
+    b = guppi_seq_scalar_max(state->y_data);
+    w = (b-a)*0.025;
+    guppi_canvas_item_set_y_bounds(GUPPI_CANVAS_ITEM(si), a-w, b+w);
+    guppi_canvas_item_home_view(GUPPI_CANVAS_ITEM(si));
+  }
 }
 
 
@@ -78,20 +93,22 @@ guppi_scatter_item_update(GnomeCanvasItem* item, double affine[6],
 			  ArtSVP* clip_path, gint flags)
 {
   GuppiScatterItem* si = GUPPI_SCATTER_ITEM(item);
+  GuppiScatterState* state =
+    GUPPI_SCATTER_STATE(guppi_canvas_item_state(GUPPI_CANVAS_ITEM(item)));
 
   /* Chain our calls to update */
   if (parent_class && GNOME_CANVAS_ITEM_CLASS(parent_class)->update)
     GNOME_CANVAS_ITEM_CLASS(parent_class)->update(item,affine,clip_path,flags);
 
-  if (si->state == NULL)
+  if (state == NULL)
     return;
 
-  if (si->state->x_data == NULL || si->state->y_data == NULL)
+  if (state->x_data == NULL || state->y_data == NULL)
     return;
 
   if (si->vis_mask)
     gtk_object_unref(GTK_OBJECT(si->vis_mask));
-  si->vis_mask = GUPPI_SEQ_BOOLEAN(guppi_seq_boolean_new_aligned(GUPPI_SEQ(si->state->x_data)));
+  si->vis_mask = GUPPI_SEQ_BOOLEAN(guppi_seq_boolean_new_aligned(GUPPI_SEQ(state->x_data)));
 
   guppi_canvas_item_request_total_redraw(GUPPI_CANVAS_ITEM(item));
 }
@@ -103,6 +120,7 @@ guppi_scatter_item_render(GnomeCanvasItem* item,
   double rx0, rx1, ry0, ry1;
   GuppiScatterItem* si = GUPPI_SCATTER_ITEM(item);
   GuppiCanvasItem* gci = GUPPI_CANVAS_ITEM(item);
+  GuppiScatterState* state = GUPPI_SCATTER_STATE(guppi_canvas_item_state(gci));
   GuppiScatterStyle* style;
   gint fuzz=5;
   gboolean got_hits = TRUE;
@@ -114,9 +132,9 @@ guppi_scatter_item_render(GnomeCanvasItem* item,
   if (parent_class && GNOME_CANVAS_ITEM_CLASS(parent_class)->render)
     GNOME_CANVAS_ITEM_CLASS(parent_class)->render(item, buf);
 
-  if (si->state == NULL)
+  if (state == NULL)
     return;
-  if (si->state->x_data == NULL || si->state->y_data == NULL)
+  if (state->x_data == NULL || state->y_data == NULL)
     return;
 
   xmag = guppi_canvas_item_x_magnification(gci);
@@ -124,9 +142,9 @@ guppi_scatter_item_render(GnomeCanvasItem* item,
   scale_factor = (xmag + ymag)/2;
 
   for (i=0; i<GUPPI_SCATTER_STATE_STYLE_COUNT; ++i) 
-    if (i == 0 || si->state->mask[i] != NULL) {
+    if (i == 0 || state->mask[i] != NULL) {
 
-      style = guppi_scatter_state_get_style(si->state, i);
+      style = guppi_scatter_state_get_style(state, i);
 
       if (style->marker == SCATTER_MARKER_ALPHA_TEMPLATE &&
 	  style->alpha_template != NULL) {
@@ -153,11 +171,11 @@ guppi_scatter_item_render(GnomeCanvasItem* item,
   /* Perform our window query and build up a bit mask indicating which
      points we need to draw. */
   guppi_seq_boolean_clear(si->vis_mask);
-  got_hits = guppi_seq_scalar_in_place_range_query(si->state->x_data,
+  got_hits = guppi_seq_scalar_in_place_range_query(state->x_data,
 						   si->vis_mask,
 						   rx0, rx1);
   if (got_hits) {
-    got_hits = guppi_seq_scalar_bitwise_and_range_query(si->state->y_data,
+    got_hits = guppi_seq_scalar_bitwise_and_range_query(state->y_data,
 							si->vis_mask,
 							ry0, ry1);
   }
@@ -171,15 +189,15 @@ guppi_scatter_item_render(GnomeCanvasItem* item,
 	 overhead of looping and testing all of the mask[i]'s (which will
 	 all be NULL) should be totally negligable. */
 
-      if (i == 0 || si->state->mask[i] != NULL) {
+      if (i == 0 || state->mask[i] != NULL) {
 	
-	style = guppi_scatter_state_get_style(si->state, i);
+	style = guppi_scatter_state_get_style(state, i);
 	if (style == NULL)
 	  g_error("Evil stalks the land");
 
-	if (si->state->mask[i]) {
+	if (state->mask[i]) {
 	  vis_mask_copy = GUPPI_SEQ_BOOLEAN(guppi_data_copy(GUPPI_DATA(si->vis_mask)));
-	  guppi_seq_boolean_bitwise_and(vis_mask_copy, si->state->mask[i]);
+	  guppi_seq_boolean_bitwise_and(vis_mask_copy, state->mask[i]);
 	} else {
 	  vis_mask_copy = si->vis_mask;
 	  gtk_object_ref(GTK_OBJECT(si->vis_mask));
@@ -191,7 +209,7 @@ guppi_scatter_item_render(GnomeCanvasItem* item,
 				  buf->rect.y1 - buf->rect.y0,
 				  buf->rect.x0, buf->rect.y0,
 				  gci->x_transform, gci->y_transform,
-				  si->state->x_data, si->state->y_data,
+				  state->x_data, state->y_data,
 				  vis_mask_copy,
 				  scale_factor);
 	gtk_object_unref(GTK_OBJECT(vis_mask_copy));
@@ -207,6 +225,39 @@ bound_hack(GnomeCanvasItem* c, double* x1, double* y1, double* x2, double* y2)
   g_assert_not_reached();
 }
 
+static void
+state_set_hook(GuppiCanvasItem* item, GuppiItemState* state)
+{
+  GuppiScatterItem* si = GUPPI_SCATTER_ITEM(item);
+  GuppiScatterState* ss = GUPPI_SCATTER_STATE(state);
+
+  g_return_if_fail(si != NULL);
+  g_return_if_fail(ss != NULL); /* The hook never gets called on NULL state */
+
+  gtk_signal_connect_object(GTK_OBJECT(ss),
+			    "reset_x_data",
+			    GTK_SIGNAL_FUNC(guppi_scatter_item_reset_x_bounds),
+			    GTK_OBJECT(si));
+
+  gtk_signal_connect_object(GTK_OBJECT(ss),
+			    "reset_y_data",
+			    GTK_SIGNAL_FUNC(guppi_scatter_item_reset_y_bounds),
+			    GTK_OBJECT(si));
+
+  gtk_signal_connect_object(GTK_OBJECT(ss),
+			    "changed_x_data",
+			    GTK_SIGNAL_FUNC(guppi_canvas_item_request_update),
+			    GTK_OBJECT(si));
+
+  gtk_signal_connect_object(GTK_OBJECT(ss),
+			    "changed_y_data",
+			    GTK_SIGNAL_FUNC(guppi_canvas_item_request_update),
+			    GTK_OBJECT(si));
+
+  guppi_scatter_item_reset_x_bounds(si);
+  guppi_scatter_item_reset_y_bounds(si);
+}
+
 static void
 guppi_scatter_item_class_init(GuppiScatterItemClass* klass)
 {
@@ -229,14 +280,14 @@ guppi_scatter_item_class_init(GuppiScatterItemClass* klass)
   item_class->update = guppi_scatter_item_update;
   item_class->render = guppi_scatter_item_render;
   item_class->bounds = bound_hack;
+
+  gci_class->state_set_hook = state_set_hook;
 }
 
 static void
 guppi_scatter_item_init(GuppiScatterItem* si)
 {
-  si->state = NULL;
   si->vis_mask = NULL;
-
 }
 
 GtkType
@@ -261,36 +312,7 @@ guppi_scatter_item_get_type(void)
   return scatter_item_type;
 }
 
-void
-guppi_scatter_item_set_state(GuppiScatterItem* si,
-			     GuppiScatterState* ss)
-{
-  g_return_if_fail(si != NULL);
-
-  /* We should do all sorts of things here: connect signals, 
-     add references, clean things up if si->state was non-null prior
-     to this assignment, etc. etc. These details can come later. */
-  si->state = ss;
-
-  guppi_canvas_item_set_state(GUPPI_CANVAS_ITEM(si),
-			      ss ? GUPPI_ITEM_STATE(ss) : NULL);
-
-  gtk_signal_connect_object(GTK_OBJECT(ss),
-			    "changed_x_data",
-			    GTK_SIGNAL_FUNC(guppi_scatter_item_reset_x_bounds),
-			    GTK_OBJECT(si));
-  gtk_signal_connect_object(GTK_OBJECT(ss),
-			    "changed_y_data",
-			    GTK_SIGNAL_FUNC(guppi_scatter_item_reset_y_bounds),
-			    GTK_OBJECT(si));
-
-  guppi_scatter_item_reset_x_bounds(si);
-  guppi_scatter_item_reset_y_bounds(si);
-  guppi_canvas_item_request_update(GUPPI_CANVAS_ITEM(si));
-
-
-}
 
 
 
-/* $Id: guppi-scatter-item.c,v 1.8 2000/02/28 05:06:17 trow Exp $ */
+/* $Id: guppi-scatter-item.c,v 1.11 2000/03/09 22:59:46 trow Exp $ */
