@@ -1,5 +1,5 @@
 /* This is -*- C++ -*- */
-/* $Id: guppi-scatter-item.h,v 1.3 2000/02/21 18:34:22 trow Exp $ */
+/* $Id: guppi-scatter-item.h,v 1.4 2000/03/09 22:59:46 trow Exp $ */
 
 /*
  * guppi-scatter-item.h
@@ -40,8 +40,6 @@ typedef struct _GuppiScatterItemClass GuppiScatterItemClass;
 struct _GuppiScatterItem {
   GuppiCanvasItem item;
 
-  GuppiScatterState* state;
-
   GuppiSeqBoolean* vis_mask;
 };
 
@@ -63,9 +61,7 @@ struct _GuppiScatterItemClass {
 
 GtkType guppi_scatter_item_get_type(void);
 
-void guppi_scatter_item_set_state(GuppiScatterItem*, GuppiScatterState*);
-
 
 #endif /* _INC_GUPPI_SCATTER-ITEM_H */
 
-/* $Id: guppi-scatter-item.h,v 1.3 2000/02/21 18:34:22 trow Exp $ */
+/* $Id: guppi-scatter-item.h,v 1.4 2000/03/09 22:59:46 trow Exp $ */
