@@ -1,5 +1,5 @@
 /* This is -*- C -*- */
-/* $Id: guppi-scatter-style.c,v 1.12 2000/02/28 05:07:34 trow Exp $ */
+/* $Id: guppi-scatter-style.c,v 1.15 2000/03/06 00:56:04 trow Exp $ */
 
 /*
  * guppi-scatter-style.c
@@ -53,6 +53,12 @@ guppi_scatter_marker_info[SCATTER_MARKER_LAST_MARKER] = {
   { SCATTER_MARKER_AST, N_("Asterisk"),
     N_("Radius"), 1, 15, 4,
     N_("Thickness"), 0.5, 12, 1 },
+  { SCATTER_MARKER_PIXEL, N_("Pixel"),
+    NULL, 0,0,0,
+    NULL, 0,0,0 },
+  { SCATTER_MARKER_DOT, N_("Dot"),
+    NULL, 0,0,0,
+    NULL, 0,0,0 },
   { SCATTER_MARKER_FILLED_CIRCLE, N_("Filled Circle"),
     N_("Radius"), 1, 15, 4,
     NULL, 0, 0, 0 },
@@ -119,6 +125,8 @@ guppi_scatter_marker_make_alpha_template(GuppiScatterMarker m,
 
   switch (m) {
   case SCATTER_MARKER_NONE:
+  case SCATTER_MARKER_PIXEL:
+  case SCATTER_MARKER_DOT:
   case SCATTER_MARKER_ALPHA_TEMPLATE:
     return NULL;
   case SCATTER_MARKER_CIRCLE:
@@ -687,6 +695,16 @@ guppi_scatter_style_paint(GuppiScatterStyle* sty,
   double size1_grad_min=0, size1_grad_max=1, size1_now;
   double size2_grad_min=0, size2_grad_max=1, size2_now;
 
+  gconstpointer x_data_raw = NULL;
+  gconstpointer y_data_raw = NULL;
+  gint x_data_stride, y_data_stride;
+  double x_val, y_val;
+
+  gconstpointer gradient_data_raw = NULL;
+  gconstpointer size1_data_raw = NULL;
+  gconstpointer size2_data_raw = NULL;
+  gint gradient_data_stride, size1_data_stride, size2_data_stride;
+
   GuppiAlphaTemplate* atemp = NULL;
   
   gint x, y;
@@ -701,6 +719,9 @@ guppi_scatter_style_paint(GuppiScatterStyle* sty,
   if (x_data == NULL || y_data == NULL)
     return;
 
+  x_data_raw = guppi_seq_scalar_raw(x_data, &x_data_stride);
+  y_data_raw = guppi_seq_scalar_raw(y_data, &y_data_stride);
+
   /* g_message("painting %dx%d area at (%d,%d)",w,h,adj_x,adj_y);*/
 
   do_gradient_coloring = (sty->gradient_data != NULL);
@@ -714,6 +735,9 @@ guppi_scatter_style_paint(GuppiScatterStyle* sty,
   size2_now = sty->size2;
 
   if (do_gradient_coloring) {
+
+    gradient_data_raw = guppi_seq_scalar_raw(sty->gradient_data,
+					     &gradient_data_stride);
     UINT_TO_RGBA(sty->color_low, &rlo, &glo, &blo, &alo);
     grad_min = guppi_seq_scalar_min(sty->gradient_data);
     grad_max = guppi_seq_scalar_max(sty->gradient_data);
@@ -722,6 +746,9 @@ guppi_scatter_style_paint(GuppiScatterStyle* sty,
   }
   
   if (do_size1_gradient) {
+    
+    size1_data_raw = guppi_seq_scalar_raw(sty->size1_data, &size1_data_stride);
+
     size1_grad_min = guppi_seq_scalar_min(sty->size1_data);
     size1_grad_max = guppi_seq_scalar_max(sty->size1_data);
 
@@ -733,6 +760,9 @@ guppi_scatter_style_paint(GuppiScatterStyle* sty,
   }
 
   if (do_size2_gradient) {
+
+    size2_data_raw = guppi_seq_scalar_raw(sty->size2_data, &size2_data_stride);
+
     size2_grad_min = guppi_seq_scalar_min(sty->size2_data);
     size2_grad_max = guppi_seq_scalar_max(sty->size2_data);
 
@@ -760,12 +790,15 @@ guppi_scatter_style_paint(GuppiScatterStyle* sty,
     atemp = guppi_scatter_marker_make_alpha_template(sty->marker,
 						     sty->size1, sty->size2,
 						     scale_factor);
-    gtk_object_ref(GTK_OBJECT(atemp));
+    if (atemp)
+      gtk_object_ref(GTK_OBJECT(atemp));
 
     if (sty->last_alpha_template) {
       gtk_object_unref(GTK_OBJECT(sty->last_alpha_template));
     }
     sty->last_alpha_template = atemp;
+    if (atemp)
+      gtk_object_ref(GTK_OBJECT(sty->last_alpha_template));
 
     sty->last_scale = scale_factor;
   }
@@ -781,14 +814,31 @@ guppi_scatter_style_paint(GuppiScatterStyle* sty,
 
   while (k <= k_max) {
 
-    x = transform_plot2pixel_unsafe(x_transform,
-				    guppi_seq_scalar_get(x_data, k));
+    if (x_data_raw && y_data_raw) {
 
-    y = transform_plot2pixel_unsafe(y_transform,
-				    guppi_seq_scalar_get(y_data, k));
+      x_val = guppi_seq_scalar_raw_get(x_data_raw, x_data_stride, k);
+      y_val = guppi_seq_scalar_raw_get(y_data_raw, y_data_stride, k);
+
+    } else {
+
+      x_val = guppi_seq_scalar_get(x_data, k);
+      y_val = guppi_seq_scalar_get(y_data, k);
+
+    }
+
+    x = transform_plot2pixel_unsafe(x_transform, x_val);
+    y = transform_plot2pixel_unsafe(y_transform, y_val);
 
     if (do_gradient_coloring) {
-      grad = guppi_seq_scalar_get(sty->gradient_data, k);
+
+      if (gradient_data_raw) {
+	grad = guppi_seq_scalar_raw_get(gradient_data_raw,
+					gradient_data_stride,
+					k);
+      } else {
+	grad = guppi_seq_scalar_get(sty->gradient_data, k);
+      }
+
       grad = (grad-grad_min)/(grad_max-grad_min);
       if (sty->reverse_color_gradient)
 	grad = 1 - grad;
@@ -800,18 +850,38 @@ guppi_scatter_style_paint(GuppiScatterStyle* sty,
     }
 
     if (do_size1_gradient) {
-      size1_now = guppi_seq_scalar_get(sty->size1_data, k);
+
+      if (size1_data_raw) {
+	size1_now = guppi_seq_scalar_raw_get(size1_data_raw,
+					     size1_data_stride,
+					     k);
+      } else {
+	size1_now = guppi_seq_scalar_get(sty->size1_data, k);
+      }
+
       size1_now = (size1_now - size1_grad_min)/(size1_grad_max-size1_grad_min);
+
       if (sty->reverse_size1_gradient)
 	size1_now = 1 - size1_now;
+
       size1_now = size1_now*sty->size1_high + (1-size1_now)*sty->size1;
     }
 
     if (do_size2_gradient) {
-      size2_now = guppi_seq_scalar_get(sty->size2_data, k);
+
+      if (size2_data_raw) {
+	size2_now = guppi_seq_scalar_raw_get(size2_data_raw,
+					     size2_data_stride,
+					     k);
+      } else {
+	size2_now = guppi_seq_scalar_get(sty->size2_data, k);
+      }
+
       size2_now = (size2_now - size2_grad_min)/(size2_grad_max-size2_grad_min);
+
       if (sty->reverse_size2_gradient)
 	size2_now = 1 - size2_now;
+
       size2_now = size2_now*sty->size2_high + (1-size2_now)*sty->size2;
     }
 
@@ -823,12 +893,15 @@ guppi_scatter_style_paint(GuppiScatterStyle* sty,
 						       size1_now,
 						       size2_now,
 						       scale_factor);
-      gtk_object_ref(GTK_OBJECT(atemp));
+      if (atemp)
+	gtk_object_ref(GTK_OBJECT(atemp));
     }
 
     if (atemp) {
       xtweak = atemp->width/2;
       ytweak = atemp->height/2;
+    } else {
+      xtweak = ytweak = 0;
     }
 
     x -= xtweak;
@@ -858,6 +931,7 @@ guppi_scatter_style_paint(GuppiScatterStyle* sty,
 	atpr_x1 = MIN(x + atemp->width - 1, w - 1);
 	atpr_y1 = MIN(y + atemp->height - 1, h - 1);
 	
+	g_assert(atemp->data);
 	atpr_ap = atemp->data + atemp->width * (atpr_y0 - y) + (atpr_x0 - x);
 	atpr_rgbp = rgb_buffer + row_stride * atpr_y0 + 3 * atpr_x0;
 	for (atpr_j=atpr_y0; atpr_j<=atpr_y1; ++atpr_j) {
@@ -876,6 +950,64 @@ guppi_scatter_style_paint(GuppiScatterStyle* sty,
 	  atpr_rgbp += row_stride;
 	}
       }
+    } else if (sty->marker == SCATTER_MARKER_PIXEL) {
+      x -= adj_x;
+      y -= adj_y;
+      if (0 <= x && x < w && 0 <= y && y < h) {
+	guchar* pt = rgb_buffer + row_stride * y + 3 * x;
+	PIXEL_RGBA(pt, rnow, gnow, bnow, anow);
+      }
+    } else if (sty->marker == SCATTER_MARKER_DOT) {
+      /* Unrolled loop to draw a 3x3 dot with soft corners */
+      gint anow2 = (anow * 0x30) >> 8;
+      gint anow3 = (anow * 0x80) >> 8;
+      guchar* pt;
+      x -= adj_x;
+      y -= adj_y;
+      pt = rgb_buffer + row_stride * (y-1) + 3 * x;
+      if (0 <= y-1 && y-1 < h) {
+
+	if (0 <= x-1 && x-1 < w) {
+	  PIXEL_RGBA(pt-3, rnow, gnow, bnow, anow2);
+	}
+	if (0 <= x && x < w) {
+	  PIXEL_RGBA(pt, rnow, gnow, bnow, anow3);
+	}
+	if (0 <= x+1 && x+1 < w) {
+	  PIXEL_RGBA(pt+3, rnow, gnow, bnow, anow2);
+	}	
+
+      }
+      pt += row_stride;
+      if (0 <= y && y < h) {
+
+	if (0 <= x-1 && x-1 < w) {
+	  PIXEL_RGBA(pt-3, rnow, gnow, bnow, anow3);
+	}
+	if (0 <= x && x < w) {
+	  PIXEL_RGBA(pt, rnow, gnow, bnow, anow);
+	}
+	if (0 <= x+1 && x+1 < w) {
+	  PIXEL_RGBA(pt+3, rnow, gnow, bnow, anow3);
+	}
+
+      }
+      pt += row_stride;
+      if (0 <= y+1 && y+1 < h) {
+
+	if (0 <= x-1 && x-1 < w) {
+	  PIXEL_RGBA(pt-3, rnow, gnow, bnow, anow2);
+	}
+	if (0 <= x && x < w) {
+	  PIXEL_RGBA(pt, rnow, gnow, bnow, anow3);
+	}
+	if (0 <= x+1 && x+1 < w) {
+	  PIXEL_RGBA(pt+3, rnow, gnow, bnow, anow2);
+	}	
+
+      }
+      
+      
     } else {
       /* do nothing */
     }
@@ -895,4 +1027,4 @@ guppi_scatter_style_paint(GuppiScatterStyle* sty,
   }
 }
 									      
-/* $Id: guppi-scatter-style.c,v 1.12 2000/02/28 05:07:34 trow Exp $ */
+/* $Id: guppi-scatter-style.c,v 1.15 2000/03/06 00:56:04 trow Exp $ */
