@@ -1,4 +1,4 @@
-/* $Id: guppi.c,v 1.16 2000/02/21 18:35:01 trow Exp $ */
+/* $Id: guppi.c,v 1.19 2000/03/08 17:33:29 trow Exp $ */
 
 /*
  * guppi.c
@@ -33,6 +33,7 @@
 #include "corba_guppi.h"
 #include <guppi-useful.h>
 #include <guppi-data-init.h>
+#include <guppi-stat-init.h>
 #include <guppi-plot-init.h>
 #include <guppi-plot-plug-in.h>
 #include "app_scm.h"
@@ -129,7 +130,7 @@ guile_terminal_cb(GtkWidget* w, gpointer data)
 static void
 hacked_scatter_plot_cb(GtkWidget* w, gpointer data)
 {
-  guppi_eval_str("(let ((s (guppi-plot-element-new \"scatter\")) (state (guppi-plot-state-new))) (guppi-plot-state-add-element state s) (guppi-plot-state-view state) (guppi-plot-element-edit s))");
+  guppi_eval_str("(let ((s (guppi-plot-element-new \"scatter\")) (reg (guppi-plot-element-new \"slinreg\")) (state (guppi-plot-state-new))) (guppi-plot-state-add-element state reg) (guppi-plot-state-add-element state s) (guppi-plot-state-view state) (guppi-plot-element-edit s) (guppi-plot-element-edit reg))");
 }
 
 static void
@@ -248,14 +249,16 @@ guppi_main_window()
   };
 
   GtkWidget* app = NULL;
+  GtkWidget* appbar = NULL;
   GtkWidget* swin = NULL;
   GtkWidget* browser = NULL;
 
   app = gnome_app_new(PACKAGE, "Guppi");
   gtk_window_set_default_size(GTK_WINDOW(app), 300, 200);
 
-  gnome_app_set_statusbar(GNOME_APP(app), 
-			  gnome_appbar_new(FALSE,TRUE,GNOME_PREFERENCES_USER));
+  appbar = gnome_appbar_new(TRUE,TRUE,GNOME_PREFERENCES_USER);
+  gnome_app_set_statusbar(GNOME_APP(app), appbar);
+  guppi_progress_register(gnome_appbar_get_progress(GNOME_APPBAR(appbar)));
 
   gnome_app_create_menus_with_data(GNOME_APP(app), app_menus, app);
   gnome_app_install_menu_hints(GNOME_APP(app), app_menus);
@@ -357,6 +360,9 @@ real_main(void* closure, int argc, char* argv[])
   guppi_splash_message(_("Initializing Utility Library"));
   guppi_useful_init();
 
+  guppi_splash_message(_("Initializing Statistics Library"));
+  guppi_stat_init();
+
   guppi_splash_message(_("Initializing Data Library"));
   guppi_data_init();
 
@@ -401,4 +407,4 @@ main(int argc, char* argv[])
 }
 
 
-/* $Id: guppi.c,v 1.16 2000/02/21 18:35:01 trow Exp $ */
+/* $Id: guppi.c,v 1.19 2000/03/08 17:33:29 trow Exp $ */
