@@ -1,4 +1,4 @@
-/* $Id: guppi-data-init.c,v 1.5 2000/02/09 21:04:56 trow Exp $ */
+/* $Id: guppi-data-init.c,v 1.6 2000/02/22 02:50:50 trow Exp $ */
 
 /*
  * guppi-data-init.c
@@ -27,6 +27,11 @@
 #include "guppi-data-init.h"
 #include "guppi-scm-data.h"
 
+#include "guppi-seq-boolean.h"
+#include "guppi-seq-date.h"
+#include "guppi-seq-scalar.h"
+#include "guppi-seq-string.h"
+
 void
 guppi_data_init(void)
 {
@@ -34,8 +39,15 @@ guppi_data_init(void)
   g_return_if_fail(!inited);
   inited = TRUE;
 
+  /* Invoke our various data types to make sure that they have been
+     created. */
+  guppi_seq_boolean_get_type();
+  guppi_seq_date_get_type();
+  guppi_seq_scalar_get_type();
+  guppi_seq_string_get_type();
+
   guppi_scm_data_init();
 }
 
 
-/* $Id: guppi-data-init.c,v 1.5 2000/02/09 21:04:56 trow Exp $ */
+/* $Id: guppi-data-init.c,v 1.6 2000/02/22 02:50:50 trow Exp $ */
