@@ -1,4 +1,4 @@
-/* $Id: guppi-canvas-item.c,v 1.9 2000/02/28 04:59:15 trow Exp $ */
+/* $Id: guppi-canvas-item.c,v 1.13 2000/03/09 22:55:47 trow Exp $ */
 
 /*
  * guppi-canvas-item.c
@@ -29,17 +29,20 @@
 #include "guppi-canvas-item.h"
 
 enum {
-  RESIZED,
-  RESCALED,
   RESCALED_X,
   RESCALED_Y,
+  REBOUNDED_X,
+  REBOUNDED_Y,
+  RENEGOTIATE_X_START,
+  RENEGOTIATE_X_DECLARE,
+  RENEGOTIATE_Y_START,
+  RENEGOTIATE_Y_DECLARE,
   LAST_SIGNAL
 };
 
 enum {
   ARG_0,
-  ARG_X1, ARG_Y1, ARG_X2, ARG_Y2,
-  ARG_XBND, ARG_YBND
+  ARG_X1, ARG_Y1, ARG_X2, ARG_Y2
 };
 
 static GtkObjectClass* parent_class = NULL;
@@ -48,6 +51,7 @@ static guint gci_signals[LAST_SIGNAL] = { 0 };
 typedef void (*GuppiSignal_NONE__DOUBLE_DOUBLE_DOUBLE_DOUBLE)(GtkObject*, double arg1, double arg2, double arg3, double arg4, gpointer user_data);
 typedef void (*GuppiSignal_NONE__DOUBLE_DOUBLE)(GtkObject*, double arg1, double arg2, gpointer user_data);
 
+/*
 static void
 guppi_marshal_NONE__DOUBLE_DOUBLE_DOUBLE_DOUBLE(GtkObject* object
 ,						GtkSignalFunc func,
@@ -63,6 +67,7 @@ guppi_marshal_NONE__DOUBLE_DOUBLE_DOUBLE_DOUBLE(GtkObject* object
 	   GTK_VALUE_DOUBLE(args[3]),
 	   func_data);
 }
+*/
 
 static void
 guppi_marshal_NONE__DOUBLE_DOUBLE(GtkObject* object,
@@ -136,14 +141,6 @@ guppi_canvas_item_set_arg(GtkObject* obj, GtkArg* arg, guint arg_id)
   case ARG_Y2:
     gci->wy1 = GTK_VALUE_DOUBLE(*arg);
     break;
-  case ARG_XBND:
-    guppi_canvas_item_set_x_bounder(gci,
-				    GUPPI_BOUNDER(GTK_VALUE_POINTER(*arg)));
-    break;
-  case ARG_YBND:
-    guppi_canvas_item_set_y_bounder(gci,
-				    GUPPI_BOUNDER(GTK_VALUE_POINTER(*arg)));
-    break;
   default:
     did_something = TRUE;
     break;
@@ -220,26 +217,6 @@ guppi_canvas_item_class_init(GuppiCanvasItemClass* klass)
   item_class = GNOME_CANVAS_ITEM_CLASS(klass);
   parent_class = gtk_type_class(GNOME_TYPE_CANVAS_ITEM);
 
-  gci_signals[RESIZED] =
-    gtk_signal_new("resized",
-		   GTK_RUN_FIRST,
-		   object_class->type,
-		   GTK_SIGNAL_OFFSET(GuppiCanvasItemClass, resized),
-		   guppi_marshal_NONE__DOUBLE_DOUBLE_DOUBLE_DOUBLE,
-		   GTK_TYPE_NONE, 4,
-		   GTK_TYPE_DOUBLE, GTK_TYPE_DOUBLE,
-		   GTK_TYPE_DOUBLE, GTK_TYPE_DOUBLE);
-
-  gci_signals[RESCALED] = 
-    gtk_signal_new("rescaled",
-		   GTK_RUN_FIRST,
-		   object_class->type,
-		   GTK_SIGNAL_OFFSET(GuppiCanvasItemClass, rescaled),
-		   guppi_marshal_NONE__DOUBLE_DOUBLE_DOUBLE_DOUBLE,
-		   GTK_TYPE_NONE, 4,
-		   GTK_TYPE_DOUBLE, GTK_TYPE_DOUBLE,
-		   GTK_TYPE_DOUBLE, GTK_TYPE_DOUBLE);
-
   gci_signals[RESCALED_X] =
     gtk_signal_new("rescaled_x",
 		   GTK_RUN_FIRST,
@@ -257,10 +234,62 @@ guppi_canvas_item_class_init(GuppiCanvasItemClass* klass)
 		   guppi_marshal_NONE__DOUBLE_DOUBLE,
 		   GTK_TYPE_NONE, 2,
 		   GTK_TYPE_DOUBLE, GTK_TYPE_DOUBLE);
+
+  gci_signals[REBOUNDED_X] =
+    gtk_signal_new("rebounded_x",
+		   GTK_RUN_FIRST,
+		   object_class->type,
+		   GTK_SIGNAL_OFFSET(GuppiCanvasItemClass, rebounded_x),
+		   guppi_marshal_NONE__DOUBLE_DOUBLE,
+		   GTK_TYPE_NONE, 2,
+		   GTK_TYPE_DOUBLE, GTK_TYPE_DOUBLE);
+
+  gci_signals[REBOUNDED_Y] =
+    gtk_signal_new("rebounded_y",
+		   GTK_RUN_FIRST,
+		   object_class->type,
+		   GTK_SIGNAL_OFFSET(GuppiCanvasItemClass, rebounded_y),
+		   guppi_marshal_NONE__DOUBLE_DOUBLE,
+		   GTK_TYPE_NONE, 2,
+		   GTK_TYPE_DOUBLE, GTK_TYPE_DOUBLE);
+
+  gci_signals[RENEGOTIATE_X_START] = 
+    gtk_signal_new("renegotiate_x_start",
+		   GTK_RUN_FIRST,
+		   object_class->type,
+		   GTK_SIGNAL_OFFSET(GuppiCanvasItemClass, renegotiate_x_start),
+		   gtk_marshal_NONE__NONE,
+		   GTK_TYPE_NONE, 0);
+
+  gci_signals[RENEGOTIATE_X_DECLARE] = 
+    gtk_signal_new("renegotiate_x_declare",
+		   GTK_RUN_FIRST,
+		   object_class->type,
+		   GTK_SIGNAL_OFFSET(GuppiCanvasItemClass, renegotiate_x_declare),
+		   guppi_marshal_NONE__DOUBLE_DOUBLE,
+		   GTK_TYPE_NONE, 2,
+		   GTK_TYPE_DOUBLE, GTK_TYPE_DOUBLE);
+
+  gci_signals[RENEGOTIATE_Y_START] = 
+    gtk_signal_new("renegotiate_y_start",
+		   GTK_RUN_FIRST,
+		   object_class->type,
+		   GTK_SIGNAL_OFFSET(GuppiCanvasItemClass, renegotiate_y_start),
+		   gtk_marshal_NONE__NONE,
+		   GTK_TYPE_NONE, 0);
+
+  gci_signals[RENEGOTIATE_Y_DECLARE] = 
+    gtk_signal_new("renegotiate_y_declare",
+		   GTK_RUN_FIRST,
+		   object_class->type,
+		   GTK_SIGNAL_OFFSET(GuppiCanvasItemClass, renegotiate_y_declare),
+		   guppi_marshal_NONE__DOUBLE_DOUBLE,
+		   GTK_TYPE_NONE, 2,
+		   GTK_TYPE_DOUBLE, GTK_TYPE_DOUBLE);
+
+  gtk_object_class_add_signals(object_class, gci_signals, LAST_SIGNAL);
     
 
-  klass->resized = NULL;
-  klass->rescaled = NULL;
   klass->rescaled_x = NULL;
   klass->rescaled_y = NULL;
 
@@ -290,51 +319,33 @@ guppi_canvas_item_class_init(GuppiCanvasItemClass* klass)
 			  GTK_TYPE_DOUBLE,
 			  GTK_ARG_READWRITE,
 			  ARG_Y2);
-  gtk_object_add_arg_type("GuppiCanvasItem::x-bounder",
-			  GTK_TYPE_POINTER,
-			  GTK_ARG_READWRITE,
-			  ARG_XBND);
-  gtk_object_add_arg_type("GuppiCanvasItem::y-bounder",
-			  GTK_TYPE_POINTER,
-			  GTK_ARG_READWRITE,
-			  ARG_YBND);
 }
 
 static void
 guppi_canvas_item_init(GuppiCanvasItem* gci)
 {
-#ifdef OLD_EVENT_HANDLER
-  static void guppi_canvas_item_event_dispatcher(GuppiCanvasItem*,
-						 GdkEvent*, gpointer);
-
-  gtk_signal_connect(GTK_OBJECT(gci),
-		     "event",
-		     GTK_SIGNAL_FUNC(guppi_canvas_item_event_dispatcher),
-		     NULL);
-#endif
-
   gci->x_transform  = transform_new();
   gci->y_transform  = transform_new();
 
-  gci->x_bounder = NULL;
-  gci->y_bounder = NULL;
-
   gci->x0 = gci->y0 = 0;
   gci->x1 = gci->y1 = 1;
 
+  gci->bb_x0 = gci->bb_y0 = 0;
+  gci->bb_x1 = gci->bb_y1 = 1;
+
   gci->full_x0 = gci->full_y0 = 0;
   gci->full_x1 = gci->full_y1 = 1;
 
   transform_set_plot_bounds(gci->x_transform, gci->x0, gci->x1);
   transform_set_plot_bounds(gci->y_transform, gci->y0, gci->y1);
 
+  gci->mag_factor = 1.0;
+
   /* Hard-wired position for testing */
   gci->wx0 = 0;
   gci->wx1 = 100;
   gci->wy0 = 0;
   gci->wy1 = 100;
-
-  gci->expecting_double_bound_change = FALSE;
 }
 
 GtkType
@@ -360,14 +371,28 @@ guppi_canvas_item_get_type(void)
 
 /*****************************************************************************/
 
+GuppiItemState*
+guppi_canvas_item_state(GuppiCanvasItem* gci)
+{
+  g_return_val_if_fail(gci != NULL, NULL);
+
+  return gci->state;
+}
+
 static void
 guppi_canvas_item_remove_state(GuppiCanvasItem* gci)
 {
+  GuppiCanvasItemClass* klass;
+
   g_return_if_fail(gci != NULL);
 
   if (gci->state == NULL)
     return;
 
+  klass = GUPPI_CANVAS_ITEM_CLASS(GTK_OBJECT(gci)->klass);
+  if (klass->state_unset_hook)
+    (klass->state_unset_hook)(gci);
+
   gtk_signal_disconnect_by_data(GTK_OBJECT(gci->state), gci);
   gtk_object_unref(GTK_OBJECT(gci->state));
   gci->state = NULL;
@@ -376,14 +401,15 @@ guppi_canvas_item_remove_state(GuppiCanvasItem* gci)
 static void 
 state_update_cb(GuppiItemState* gis, gpointer data)
 {
-  g_return_if_fail(gis != NULL);
   g_return_if_fail(data != NULL);
   guppi_canvas_item_request_update(GUPPI_CANVAS_ITEM(data));
 }
 
+
 void
 guppi_canvas_item_set_state(GuppiCanvasItem* gci, GuppiItemState* state)
 {
+  GuppiCanvasItemClass* klass;
   g_return_if_fail(gci != NULL);
 
   if (state == gci->state)
@@ -393,98 +419,55 @@ guppi_canvas_item_set_state(GuppiCanvasItem* gci, GuppiItemState* state)
     guppi_canvas_item_remove_state(gci);
 
   gci->state = state;
+  gtk_object_ref(GTK_OBJECT(state));
 
-  gtk_signal_connect(GTK_OBJECT(gci->state),
-		     "update",
-		     GTK_SIGNAL_FUNC(state_update_cb),
-		     gci);
+  if (gci->state) {
+
+    klass = GUPPI_CANVAS_ITEM_CLASS(GTK_OBJECT(gci)->klass);
+    if (klass->state_set_hook)
+      (klass->state_set_hook)(gci, state);
+
+    gtk_signal_connect(GTK_OBJECT(gci->state),
+		       "changed",
+		       GTK_SIGNAL_FUNC(state_update_cb),
+		       gci);
+  }
+
+  guppi_canvas_item_request_update(gci);
 }
 
 /*****************************************************************************/
 
-static void
-guppi_canvas_item_set_x_scale_lowlevel(GuppiCanvasItem* gci,
-				       double x0, double x1,
-				       gboolean redraw)
+void
+guppi_canvas_item_set_x_scale(GuppiCanvasItem* gci, double x0, double x1)
 {
+  g_return_if_fail(gci);
+
   if (gci->x0 == x0 && gci->x1 == x1)
     return;
 
   gci->x0 = x0;
   gci->x1 = x1;
-
-  transform_set_plot_bounds(gci->x_transform, gci->x0, gci->x1);
+  transform_set_plot_bounds(gci->x_transform, x0, x1);
 
   gtk_signal_emit(GTK_OBJECT(gci), gci_signals[RESCALED_X], x0, x1);
- 
-  if (redraw)
-    guppi_canvas_item_request_update(gci);
-}
-
-static void
-guppi_canvas_item_set_y_scale_lowlevel(GuppiCanvasItem* gci,
-				       double y0, double y1,
-				       gboolean redraw)
-{
-  if (gci->y0 == y0 && gci->y1 == y1)
-    return;
-
-  gci->y0 = y0;
-  gci->y1 = y1;
-
-  transform_set_plot_bounds(gci->y_transform, gci->y0, gci->y1);
-
-  gtk_signal_emit(GTK_OBJECT(gci), gci_signals[RESCALED_Y], y0, y1);
- 
-  if (redraw)
-    guppi_canvas_item_request_update(gci);
-}
-
-
-void
-guppi_canvas_item_set_x_scale(GuppiCanvasItem* gci, double x0, double x1)
-{
-  double w;
-
-  g_return_if_fail(gci);
-
-  if (gci->x_bounder) {
-
-    /* Pass our request onto the bounder */
-    w = gci->full_x1 - gci->full_x0;
-    guppi_bounder_set(gci->x_bounder,
-		      (x0 - gci->full_x0) / w,
-		      (x1 - gci->full_x0) / w);
-
-  } else {
-
-    guppi_canvas_item_set_x_scale_lowlevel(gci, x0, x1, TRUE);
-
-  }
+  guppi_canvas_item_request_update(gci);
 }
 
 void
 guppi_canvas_item_set_y_scale(GuppiCanvasItem* gci, double y0, double y1)
 {
-  double w;
-
   g_return_if_fail(gci);
 
-  if (gci->y_bounder) {
-
-    /* Pass our request onto the bounder */
-    w = gci->full_y1 - gci->full_y0;
-    guppi_bounder_set(gci->y_bounder,
-		      (y0 - gci->full_y0) / w,
-		      (y1 - gci->full_y0) / w);
-
-  } else {
-
-    guppi_canvas_item_set_y_scale_lowlevel(gci, y0, y1, TRUE);
-
-  }
+  if (gci->y0 == y0 && gci->y1 == y1)
+    return;
 
+  gci->y0 = y0;
+  gci->y1 = y1;
+  transform_set_plot_bounds(gci->y_transform, y0, y1);
 
+  gtk_signal_emit(GTK_OBJECT(gci), gci_signals[RESCALED_Y], y0, y1);
+  guppi_canvas_item_request_update(gci);
 }
 
 void
@@ -494,46 +477,8 @@ guppi_canvas_item_set_scales(GuppiCanvasItem* gci,
 {
   g_return_if_fail(gci);
 
-  /* Our goal to avoid unnecessary redraw operations. */
-
-  if (gci->x0 == x0 && gci->x1 == x1 && gci->y0 == y0 && gci->y1 == y1)
-    return;
-
-  if (gci->x0 == x0 && gci->x1 == x1) {
-    guppi_canvas_item_set_y_scale(gci, y0, y1);
-    return;
-  }
-
-  if (gci->y0 == y0 && gci->y1 == y1) {
-    guppi_canvas_item_set_x_scale(gci, x0, x1);
-    return;
-  }
-
-  if (gci->x_bounder && gci->y_bounder) {
-
-    gci->expecting_double_bound_change = TRUE;
-    guppi_canvas_item_set_x_scale(gci, x0, x1);
-    guppi_canvas_item_set_y_scale(gci, y0, y1);
-
-  } else if (gci->x_bounder) {
-
-    guppi_canvas_item_set_y_scale_lowlevel(gci, y0, y1, FALSE);
-    guppi_canvas_item_set_x_scale(gci, x0, x1);
-
-  } else if (gci->y_bounder) {
-
-    guppi_canvas_item_set_x_scale_lowlevel(gci, x0, x1, FALSE);
-    guppi_canvas_item_set_y_scale(gci, y0, y1);
-
-  } else {
-
-    guppi_canvas_item_set_x_scale_lowlevel(gci, x0, x1, FALSE);
-    guppi_canvas_item_set_y_scale_lowlevel(gci, y0, y1, TRUE);
-
-  }
-
-  gtk_signal_emit(GTK_OBJECT(gci), gci_signals[RESCALED], x0, y0, x1, y1);
-  guppi_canvas_item_request_update(gci);
+  guppi_canvas_item_set_x_scale(gci, x0, x1);
+  guppi_canvas_item_set_y_scale(gci, y0, y1);
 }
 
 /*****************************************************************************/
@@ -578,54 +523,142 @@ guppi_canvas_item_translate(GuppiCanvasItem* gci, double dx, double dy)
 			       gci->x1 + dx, gci->y1 + dy);
 }
 
+void
+guppi_canvas_item_home_view(GuppiCanvasItem* gci)
+{
+  g_return_if_fail(gci != NULL);
+
+  guppi_canvas_item_set_scales(gci,
+			       gci->full_x0, gci->full_y0,
+			       gci->full_x1, gci->full_y1);
+}
+
 /*****************************************************************************/
 
-/*
+/* Plot coordinate negotiations */
 
-  We have to follow our calls to set_{x,y}_scale() by a
-  set_{x,y}_scale_lowlevel() to force updating in certain situations.
-  If they are redundant, they will just return without doing anything,
-  so they are harmless... no extra redraws or anything should result.
+static void
+renegotiate_x_start(GuppiCanvasItem* gci)
+{
+  if (gci->reneg_x_started)
+    return;
+  gci->full_x0 = gci->bb_x0;
+  gci->full_x1 = gci->bb_x1;
+  gci->reneg_x_started = TRUE;
+  gtk_signal_emit(GTK_OBJECT(gci), gci_signals[RENEGOTIATE_X_START]);
+  if (!gci->bb_passive)
+    gtk_signal_emit(GTK_OBJECT(gci), gci_signals[RENEGOTIATE_X_DECLARE],
+		    gci->full_x0, gci->full_x1);
+}
 
-  Here is the problem:
+static void
+renegotiate_x_accept(GuppiCanvasItem* src, double x0, double x1,
+		     GuppiCanvasItem* gci)
+{
+  double m0, m1;
 
-  Say we start out with bounds [0,10] and scale [0,10].
-  Thus the bounder is at [0,1].
+  if (gci->bb_passive && gci->full_x0 == gci->bb_x0 && gci->full_x1 == gci->bb_x1) {
+    m0 = x0;
+    m1 = x1;
+  } else {
+    m0 = MIN(gci->full_x0, x0);
+    m1 = MIN(gci->full_x1, x1);
+  }
 
-  We increase the bounds to [0,20], and we automatically call
-  set_scale to adjust our scale to [0,20].  This forwards a request to
-  the bounder, requesting that it set itself to [0,1].  Since this is
-  the same as before, the bounder drops the request as redundant.
-  Since no bounder-changed signal is ever emitted, our set_*_scale
-  call never results in a set_*_scale_lowevel call, even though our
-  scale has certainly changed.
+  /* This flag needs to get reset somewhere, after all of the _start
+     functions are finished. */
+  gci->reneg_x_started = FALSE; 
 
-  By explicitly forcing a call to lowlevel, we should (hopefully!)
-  avoid these problems.
+  if (gci->full_x0 == m0 && gci->full_x1 == m1)
+    return;
+  
+  gci->full_x0 = m0;
+  gci->full_x1 = m1;
+  
+  /* If anything changed, we re-declare */
+  gtk_signal_emit(GTK_OBJECT(gci), gci_signals[RENEGOTIATE_X_DECLARE],
+		  gci->full_x0, gci->full_x1);
 
-  */
+  guppi_canvas_item_request_update(gci);
+}
+
+static void
+renegotiate_y_start(GuppiCanvasItem* gci)
+{
+  if (gci->reneg_y_started)
+    return;
+  gci->full_y0 = gci->bb_y0;
+  gci->full_y1 = gci->bb_y1;
+  gci->reneg_y_started = TRUE;
+
+  gtk_signal_emit(GTK_OBJECT(gci), gci_signals[RENEGOTIATE_Y_START]);
+  if (!gci->bb_passive)
+    gtk_signal_emit(GTK_OBJECT(gci), gci_signals[RENEGOTIATE_Y_DECLARE],
+		    gci->full_y0, gci->full_y1);
+}
+
+static void
+renegotiate_y_accept(GuppiCanvasItem* src, double y0, double y1,
+		     GuppiCanvasItem* gci)
+{
+  double m0, m1;
+
+  if (gci->bb_passive && gci->full_y0 == gci->bb_y0 && gci->full_y1 == gci->bb_y1) {
+    m0 = y0;
+    m1 = y1;
+  } else {
+    m0 = MIN(gci->full_y0, y0);
+    m1 = MIN(gci->full_y1, y1);
+  }
+
+  /* This flag needs to get reset somewhere, after all of the _start
+     functions are finished. */
+  gci->reneg_y_started = FALSE; 
+
+  if (gci->full_y0 == m0 && gci->full_y1 == m1)
+    return;
+
+  gci->full_y0 = m0;
+  gci->full_y1 = m1;
+  
+  /* If anything changed, we re-declare */
+  gtk_signal_emit(GTK_OBJECT(gci), gci_signals[RENEGOTIATE_Y_DECLARE],
+		  gci->full_y0, gci->full_y1);
+
+  guppi_canvas_item_request_update(gci);
+}
+
+/*****************************************************************************/
 
 void
 guppi_canvas_item_set_x_bounds(GuppiCanvasItem* gci, double x0, double x1)
 {
   g_return_if_fail(gci != NULL);
-  gci->full_x0 = x0;
-  gci->full_x1 = x1;
-  guppi_canvas_item_set_x_scale(gci, x0, x1);
-  guppi_canvas_item_set_x_scale_lowlevel(gci, x0, x1, TRUE);
 
-   guppi_canvas_item_request_update(gci);
+  if (gci->bb_x0 == x0 && gci->bb_x1 == x1)
+    return;
+
+  gci->bb_x0 = x0;
+  gci->bb_x1 = x1;
+
+  gtk_signal_emit(GTK_OBJECT(gci), gci_signals[REBOUNDED_X], x0, x1);  
+  renegotiate_x_start(gci);
+  guppi_canvas_item_request_update(gci);
 }
 
 void
 guppi_canvas_item_set_y_bounds(GuppiCanvasItem* gci, double y0, double y1)
 {
   g_return_if_fail(gci != NULL);
-  gci->full_y0 = y0;
-  gci->full_y1 = y1;
-  guppi_canvas_item_set_y_scale(gci, y0, y1);
-  guppi_canvas_item_set_y_scale_lowlevel(gci, y0, y1, TRUE);
 
+  if (gci->bb_y0 == y0 && gci->bb_y1 == y1)
+    return;
+
+  gci->bb_y0 = y0;
+  gci->bb_y1 = y1;
+
+  gtk_signal_emit(GTK_OBJECT(gci), gci_signals[REBOUNDED_Y], y0, y1);  
+  renegotiate_y_start(gci);
   guppi_canvas_item_request_update(gci);
 }
 
@@ -636,112 +669,8 @@ guppi_canvas_item_set_bounds(GuppiCanvasItem* gci,
 {
   g_return_if_fail(gci != NULL);
 
-  gci->full_x0 = x0;
-  gci->full_x1 = x1;
-  gci->full_y0 = y0;
-  gci->full_y1 = y1;
-
-  guppi_canvas_item_set_scales(gci, x0, y0, x1, y1);
-  guppi_canvas_item_set_x_scale_lowlevel(gci, x0, x1, FALSE);
-  guppi_canvas_item_set_y_scale_lowlevel(gci, y0, y1, TRUE);
-
-  guppi_canvas_item_request_update(gci);
-}
-
-/*****************************************************************************/
-
-static void
-gci_x_bounder_cb(GuppiBounder* gb, gpointer user_data)
-{
-  GuppiCanvasItem* gci = GUPPI_CANVAS_ITEM(user_data);
-  double nx0, nx1;
-  double w;
-
-  g_return_if_fail(gb != NULL);
-  g_return_if_fail(gci != NULL);
-
-  w = gci->full_x1 - gci->full_x0;
-  nx0 = gci->full_x0 + w * gb->t0;
-  nx1 = gci->full_x0 + w * gb->t1;
-
-  guppi_canvas_item_set_x_scale_lowlevel(gci, nx0, nx1,
-					 !gci->expecting_double_bound_change);
-
-  gci->expecting_double_bound_change = FALSE;
-}
-
-static void
-gci_y_bounder_cb(GuppiBounder* gb, gpointer user_data)
-{
-  GuppiCanvasItem* gci = GUPPI_CANVAS_ITEM(user_data);
-  double ny0, ny1;
-  double w;
-
-  g_return_if_fail(gb != NULL);
-  g_return_if_fail(gci != NULL);
-
-  w = gci->full_y1 - gci->full_y0;
-  ny0 = gci->full_y0 + w * gb->t0;
-  ny1 = gci->full_y0 + w * gb->t1;
-
-  guppi_canvas_item_set_y_scale_lowlevel(gci, ny0, ny1,
-					 !gci->expecting_double_bound_change);
-
-  gci->expecting_double_bound_change = FALSE;
-}
-
-void
-guppi_canvas_item_set_x_bounder(GuppiCanvasItem* gci, GuppiBounder* gb)
-{
-  g_return_if_fail(gci != NULL);
-
-  if (gci->x_bounder == gb)
-    return;
-
-  if (gci->x_bounder) {
-    gtk_object_unref(GTK_OBJECT(gci->x_bounder));
-    gtk_signal_disconnect(GTK_OBJECT(gci->x_bounder), gci->x_bnd_sh);
-  }
-
-  gci->x_bounder = gb;
-    
-  if (gb) {
-    gtk_object_ref(GTK_OBJECT(gci->x_bounder));
-    gci->x_bnd_sh = gtk_signal_connect(GTK_OBJECT(gci->x_bounder),
-				       "changed",
-				       GTK_SIGNAL_FUNC(gci_x_bounder_cb),
-				       (gpointer)gci);
-  }
-
-  guppi_canvas_item_request_update(gci);
-
-}
-
-void
-guppi_canvas_item_set_y_bounder(GuppiCanvasItem* gci, GuppiBounder* gb)
-{
-  g_return_if_fail(gci != NULL);
-
-  if (gci->y_bounder == gb)
-    return;
-
-  if (gci->y_bounder) {
-    gtk_object_unref(GTK_OBJECT(gci->y_bounder));
-    gtk_signal_disconnect(GTK_OBJECT(gci->y_bounder), gci->y_bnd_sh);
-  }
-
-  gci->y_bounder = gb;
-    
-  if (gb) {
-    gtk_object_ref(GTK_OBJECT(gci->y_bounder));
-    gci->y_bnd_sh = gtk_signal_connect(GTK_OBJECT(gci->y_bounder),
-				       "changed",
-				       GTK_SIGNAL_FUNC(gci_y_bounder_cb),
-				       (gpointer)gci);
-  }
-
-  guppi_canvas_item_request_update(gci);
-
+  guppi_canvas_item_set_x_bounds(gci, x0, x1);
+  guppi_canvas_item_set_y_bounds(gci, y0, y1);
 }
 
 /*****************************************************************************/
@@ -749,17 +678,39 @@ guppi_canvas_item_set_y_bounder(GuppiCanvasItem* gci, GuppiBounder* gb)
 double
 guppi_canvas_item_x_magnification(const GuppiCanvasItem* gci)
 {
+  double mag;
+
   g_return_val_if_fail(gci != NULL, 0);
 
-  return fabs( (gci->full_x1 - gci->full_x0) / (gci->x1 - gci->x0) );
+  mag = gci->mag_factor * fabs( (gci->full_x1-gci->full_x0)/(gci->x1-gci->x0) );
+
+  return mag;
 }
 
 double
 guppi_canvas_item_y_magnification(const GuppiCanvasItem* gci)
 {
+  double mag;
+
   g_return_val_if_fail(gci != NULL, 0);
 
-  return fabs( (gci->full_y1 - gci->full_y0) / (gci->y1 - gci->y0) );
+  mag = gci->mag_factor * fabs( (gci->full_y1-gci->full_y0)/(gci->y1-gci->y0) );
+
+  return mag;
+
+}
+
+double
+guppi_canvas_item_magnification(const GuppiCanvasItem* gci)
+{
+  double mx, my;
+
+  g_return_val_if_fail(gci != NULL, 0);
+
+  mx = guppi_canvas_item_x_magnification(gci);
+  my = guppi_canvas_item_y_magnification(gci);
+
+  return (mx+my)/2;
 }
 
 /*****************************************************************************/
@@ -813,7 +764,7 @@ guppi_canvas_item_plot2world(const GuppiCanvasItem* gci,
   g_return_if_fail(gci != NULL);
 
   guppi_canvas_item_plot2pixel(gci, x, y, &cx, &cy);
-  gnome_canvas_c2w(GNOME_CANVAS_ITEM(gci)->canvas, cy, cy, wx, wy);
+  gnome_canvas_c2w(GNOME_CANVAS_ITEM(gci)->canvas, cx, cy, wx, wy);
 }
 
 void
@@ -855,6 +806,124 @@ guppi_canvas_item_request_update(GuppiCanvasItem* gci)
 
 /***************************************************************************/
 
-/* $Id: guppi-canvas-item.c,v 1.9 2000/02/28 04:59:15 trow Exp $ */
+static void
+rescale_x_cb(GuppiCanvasItem* a, double x0, double x1, GuppiCanvasItem* b)
+{
+  guppi_canvas_item_set_x_scale(b, x0, x1);
+}
+
+static void
+rescale_y_cb(GuppiCanvasItem* a, double y0, double y1, GuppiCanvasItem* b)
+{
+  guppi_canvas_item_set_y_scale(b, y0, y1);
+}
+
+void
+guppi_canvas_item_connect_x_scales(GuppiCanvasItem* a, GuppiCanvasItem* b)
+{
+  g_return_if_fail(a != NULL);
+  g_return_if_fail(b != NULL);
+
+  gtk_signal_connect(GTK_OBJECT(a),
+		     "rescaled_x",
+		     GTK_SIGNAL_FUNC(rescale_x_cb),
+		     b);
+
+  gtk_signal_connect_object(GTK_OBJECT(a),
+			    "renegotiate_x_start",
+			    GTK_SIGNAL_FUNC(renegotiate_x_start),
+			    GTK_OBJECT(b));
+
+  gtk_signal_connect(GTK_OBJECT(a),
+		     "renegotiate_x_declare",
+		     GTK_SIGNAL_FUNC(renegotiate_x_accept),
+		     b);
+
+  gtk_signal_connect(GTK_OBJECT(b),
+		     "rescaled_x",
+		     GTK_SIGNAL_FUNC(rescale_x_cb),
+		     a);
+
+  gtk_signal_connect_object(GTK_OBJECT(b),
+			    "renegotiate_x_start",
+			    GTK_SIGNAL_FUNC(renegotiate_x_start),
+			    GTK_OBJECT(a));
+
+  gtk_signal_connect(GTK_OBJECT(b),
+		     "renegotiate_x_declare",
+		     GTK_SIGNAL_FUNC(renegotiate_x_accept),
+		     a);
+
+}
+
+void
+guppi_canvas_item_connect_y_scales(GuppiCanvasItem* a, GuppiCanvasItem* b)
+{
+  g_return_if_fail(a != NULL);
+  g_return_if_fail(b != NULL);
+
+  gtk_signal_connect(GTK_OBJECT(a),
+		     "rescaled_y",
+		     GTK_SIGNAL_FUNC(rescale_y_cb),
+		     b);
+
+  gtk_signal_connect_object(GTK_OBJECT(a),
+			    "renegotiate_y_start",
+			    GTK_SIGNAL_FUNC(renegotiate_y_start),
+			    GTK_OBJECT(b));
+
+  gtk_signal_connect(GTK_OBJECT(a),
+		     "renegotiate_y_declare",
+		     GTK_SIGNAL_FUNC(renegotiate_y_accept),
+		     b);
+
+  gtk_signal_connect(GTK_OBJECT(b),
+		     "rescaled_y",
+		     GTK_SIGNAL_FUNC(rescale_y_cb),
+		     a);
+
+  gtk_signal_connect_object(GTK_OBJECT(b),
+			    "renegotiate_y_start",
+			    GTK_SIGNAL_FUNC(renegotiate_y_start),
+			    GTK_OBJECT(a));
+
+  gtk_signal_connect(GTK_OBJECT(b),
+		     "renegotiate_y_declare",
+		     GTK_SIGNAL_FUNC(renegotiate_y_accept),
+		     a);
+
+}
+
+void
+guppi_canvas_item_connect_scales(GuppiCanvasItem* a, GuppiCanvasItem* b)
+{
+  guppi_canvas_item_connect_x_scales(a, b);
+  guppi_canvas_item_connect_y_scales(a, b);
+}
+
+/****************************************************************************/
+
+void
+guppi_canvas_item_disconnect(GuppiCanvasItem* a, GuppiCanvasItem* b)
+{
+  gtk_signal_disconnect_by_func(GTK_OBJECT(a),
+				GTK_SIGNAL_FUNC(rescale_x_cb),
+				b);
+
+  gtk_signal_disconnect_by_func(GTK_OBJECT(b),
+				GTK_SIGNAL_FUNC(rescale_x_cb),
+				a);
+
+  gtk_signal_disconnect_by_func(GTK_OBJECT(a),
+				GTK_SIGNAL_FUNC(rescale_y_cb),
+				b);
+
+  gtk_signal_disconnect_by_func(GTK_OBJECT(b),
+				GTK_SIGNAL_FUNC(rescale_y_cb),
+				a);
+
+}
+
+/* $Id: guppi-canvas-item.c,v 1.13 2000/03/09 22:55:47 trow Exp $ */
 
 
