@@ -1,5 +1,5 @@
 /* This is -*- C -*- */
-/* $Id: guppi-plot-state.c,v 1.5 2000/02/18 06:49:40 trow Exp $ */
+/* $Id: guppi-plot-state.c,v 1.8 2000/03/09 02:57:24 trow Exp $ */
 
 /*
  * guppi-plot-state.c
@@ -245,37 +245,13 @@ item_raise_lower_cb(GuppiPlotElement* e, gint n, gpointer data)
     gnome_canvas_item_lower(GNOME_CANVAS_ITEM(data), -n);
 }
 
-static void
-item_update_cb(GuppiPlotElement* e, gpointer data)
-{
-  static GnomeCanvasItem* gci = NULL;
-
-  if (gci == NULL && FALSE) {
-    gci = gnome_canvas_item_new(gnome_canvas_root(GNOME_CANVAS_ITEM(data)->canvas),
-				gnome_canvas_rect_get_type(),
-				"x1", 50.0,
-				"y1", 50.0,
-				"x2", 200.0,
-				"y2", 200.0,
-				"fill_color_rgba", 0x00ffff9f,
-				"outline_color", "black",
-				"width_units", 4.0,
-				NULL);
-  }
-
-  g_message("requesting update");
-  guppi_canvas_item_request_update(GUPPI_CANVAS_ITEM(data));
-  if (gci)
-    gnome_canvas_item_request_update(gci);
-  guppi_canvas_item_request_total_redraw(GUPPI_CANVAS_ITEM(data));
-}
-
 void
 guppi_plot_state_render(GuppiPlotState* gps, GnomeCanvas* canvas)
 {
   gint i;
   GuppiPlotElement* e;
   GuppiCanvasItem* ci;
+  GuppiCanvasItem* first_ci = NULL;
 
   g_return_if_fail(gps != NULL);
   g_return_if_fail(canvas != NULL);
@@ -291,6 +267,16 @@ guppi_plot_state_render(GuppiPlotState* gps, GnomeCanvas* canvas)
     --i;
     e = guppi_plot_state_get_element(gps, i);
     ci = guppi_plot_element_make_canvas_item(e, canvas, 0, 0, 500, 400);
+
+    /* Automatically connect all items.  This is a hack. */
+    if (first_ci == NULL)
+      first_ci = ci;
+    else {
+      guppi_canvas_item_connect_scales(first_ci, ci);
+      guppi_item_state_connect_x_data(first_ci->state, ci->state);
+      guppi_item_state_connect_y_data(first_ci->state, ci->state);
+    }
+
     gtk_signal_connect_object(GTK_OBJECT(e),
 			      "destroy",
 			      GTK_SIGNAL_FUNC(gtk_object_destroy),
@@ -307,12 +293,6 @@ guppi_plot_state_render(GuppiPlotState* gps, GnomeCanvas* canvas)
 		       "raise_or_lower",
 		       GTK_SIGNAL_FUNC(item_raise_lower_cb),
 		       ci);
-    /*
-    gtk_signal_connect(GTK_OBJECT(e),
-		       "update",
-		       GTK_SIGNAL_FUNC(item_update_cb),
-		       ci);
-    */
 
     /* If this canvas item goes away for some reason, we need to make sure
        that we are not trying to send it signals after it has been
@@ -342,4 +322,4 @@ guppi_plot_state_view(GuppiPlotState* state)
 }
 
 
-/* $Id: guppi-plot-state.c,v 1.5 2000/02/18 06:49:40 trow Exp $ */
+/* $Id: guppi-plot-state.c,v 1.8 2000/03/09 02:57:24 trow Exp $ */
