@@ -1,10 +1,10 @@
-/* This is -*- C++ -*- */
-/* $Id: jitter.h,v 1.1.1.1 1999/12/03 07:02:53 trow Exp $ */
+/* This is -*- C -*- */
+/* $Id: jitter.h,v 1.2 2000/02/06 03:17:13 trow Exp $ */
 
 /*
  * jitter.h
  *
- * Copyright (C) 1999 EMC Capital Management, Inc.
+ * Copyright (C) 1999, 2000 EMC Capital Management, Inc.
  *
  * Developed by Jon Trowbridge <trow@gnu.org>
  * and Havoc Pennington <hp@pobox.com>.
@@ -28,12 +28,13 @@
 #ifndef _INC_JITTER_H
 #define _INC_JITTER_H
 
-#include <guppi-data.h>
+#include <config.h>
+#include <gnome.h>
 
 /* Return "quasi-random" (x,y) values s.t. |(x,y)| <= 1 */
-void jitter(guint seed, double size, dindex_t i, double* xj, double* yj);
+void jitter(guint seed, double size, gint i, double* xj, double* yj);
 
 
 #endif /* _INC_JITTER_H */
 
-/* $Id: jitter.h,v 1.1.1.1 1999/12/03 07:02:53 trow Exp $ */
+/* $Id: jitter.h,v 1.2 2000/02/06 03:17:13 trow Exp $ */
