@@ -1,4 +1,4 @@
-/* $Id: guppi-guile.h,v 1.1 2000/01/17 05:44:56 trow Exp $ */
+/* $Id: guppi-guile.h,v 1.3 2000/02/10 02:07:03 trow Exp $ */
 
 /* 
  * guppi-guile.h
@@ -93,6 +93,17 @@ gboolean guppi_enum_val_to_string(const ScmEnumInfo* info, gint val, const char*
 void guppi_scm_thunk_cb(gpointer widget, gpointer thunk);
 void guppi_scm_eval_cb(gpointer widget, gpointer eval_str);
 
+/* Some convenient type conversion gadgets */
+
+gboolean scm_stringv_p(SCM);
+SCM stringv2scm(gchar**);
+gchar** scm2stringv(SCM);
+
+gboolean scm_color_p(SCM);
+SCM color2scm(guint32);
+guint32 scm2color(SCM);
+
+
 #endif /* _INC_GUPPI_GUILE_H */
 
-/* $Id: guppi-guile.h,v 1.1 2000/01/17 05:44:56 trow Exp $ */
+/* $Id: guppi-guile.h,v 1.3 2000/02/10 02:07:03 trow Exp $ */
