@@ -1,4 +1,4 @@
-/* $Id: guppi-paths.c,v 1.3 2000/02/16 22:47:53 trow Exp $ */
+/* $Id: guppi-paths.c,v 1.4 2000/03/05 23:43:38 trow Exp $ */
 
 /*
  * guppi-paths.c
@@ -73,6 +73,12 @@ guppi_glade_path(const gchar* filename)
   if (g_file_exists(resolved))
     return resolved;
 
+  /* This is really bad... */
+  g_free(resolved);
+  resolved = g_strdup_printf("../../plug-ins/slinreg/%s", filename);
+  if (g_file_exists(resolved))
+    return resolved;
+
   g_free(resolved);
   resolved = g_strdup_printf("../../plug-ins/import-delimited/%s", filename);
   if (g_file_exists(resolved))
@@ -121,4 +127,4 @@ gnome_guile_repl_path(void)
 
 
 
-/* $Id: guppi-paths.c,v 1.3 2000/02/16 22:47:53 trow Exp $ */
+/* $Id: guppi-paths.c,v 1.4 2000/03/05 23:43:38 trow Exp $ */
